#if !defined HAVE_PERMSTAR_H__
#define      HAVE_PERMSTAR_H__


#include "fxttypes.h"
#include "inline.h"
#include "newop.h"


class perm_star
//
// algorithm of Gideon Ehrlich, as given in Knuth
//
{
public:
    ulong n_;
    ulong swp_;
    ulong *a_;
    ulong *b_;
    ulong *c_;

public:
    perm_star(ulong n)
        : n_(n)
    {
        a_ = NEWOP(ulong, 3*n_+1);
        b_ = a_ + n_;
        c_ = b_ + n_;
        init();
    }

    ~perm_star()  { delete [] a_; }

    void init()
    {
        swp_ = 0;
        for (ulong k=0; k<n_; ++k)  a_[k] = b_[k] = k;
        for (ulong k=0; k<=n_; ++k)  c_[k] = 0;
    }

    ulong next()
    {
        ulong k = 1;

        if ( c_[k] == k )
        {
            do
            {
                c_[k] = 0;
                ++k;
            }
            while ( c_[k] >= k );
        }

        if ( k == n_ )  return 0;
        else            ++c_[k];

        swp_ = b_[k];
        swap(a_[0], a_[swp_]);

        ulong j = 1;
        --k;

        while ( j < k )
        {
            swap(b_[j], b_[k]);
            ++j;
            --k;
        }

        return  1;  // ==0
    }

    ulong get_swap()  const  { return swp_; }

//    ulong current()  const  { return k_; }

    const ulong *data()  const  { return a_; }
};
// -------------------------


#endif  // !defined HAVE_PERMSTAR_H__
